"""
Name: pycord-btns-menus

Version: 0.1.7

Summary: A responsive package for Buttons, DropMenus and Combinations

Home-page: https://github.com/skrphenix/pycord_btns_menus

Author: P. Sai Keerthan Reddy

Author-email: saikeerthan.keerthan.9@gmail.com

License: MIT

Location: c:/users/sai keerthan/appdata/local/programs/python/python39/lib/site-packages

Requires: py-cord
"""
